/* filter.c - The filter-program
 *            MD 10/90
 *            MD 5/91 - Extended to allow "unescaped" translation
 */
 
#include <stdio.h>
#include <string.h> 

#define MAXFLTLEN 2000

int ignore,noeof;
char escape;
char buff[MAXFLTLEN];
size_t filter_len;
char outdev[64]="PRN";
char filter[64]="FILTER.FLT";
char infname[64]="";

void do_filter(FILE *in,FILE *out)
{
  char *seqhash[256];
  char *seqhashesc[256];
  char *fptr,*escptr;
  int i,inesc=0,count,c;
  
  for(i=0;i<256;i++)
  {
    seqhash[i]=0;
    seqhashesc[i]=0;
  }
  fptr=buff;
  do
  {
    if(*fptr)
      seqhash[*fptr]=fptr+1;
    else
    {
      fptr++;
      seqhashesc[*fptr]=fptr+1;
    }
    fptr++;
    fptr+=*fptr+1;
  }
  while(fptr<buff+filter_len);
  while((c=fgetc(in))!=EOF)
  {
    if(inesc)
    {
      if(seqhashesc[c])
      {
        if(!ignore)
        {
          count=seqhashesc[c][0];
          escptr=seqhashesc[c]+1;
          while(count--)
            fputc(*escptr++,out);
        }
      }
      else
        fputc(c,out);
      inesc=0;
    }
    else
    {
      if(c==escape)
        inesc=1;
      else
        if(seqhash[c])
        {
          count=seqhash[c][0];
          escptr=seqhash[c]+1;
          while(count--)
            fputc(*escptr++,out);
        }
        else
          fputc(c,out);
    }
  }
  if(!noeof)
    fputc(26,out); /* ^Z if it is necessary */
}

int main(int argc,char **argv)
{
  FILE *in,*out,*flt;
  int i;

  if(argc<2)
  {
    puts("Benutzung: {Optionen} <Dateiname> {<Ausgabedatei>}");
    puts("Zweck    : Schreibt eine anhand der Filterdatei (normal: FILTER.FLT)");
    puts("           bersetzte Datei <Dateiname> in die Datei <Ausgabedatei>,");
    puts("           die wenn nicht angegeben als PRN: (Drucker) angenommen wird.");
    puts("Optionen : -i  - Ignoriert die Escape Sequenzen");
    puts("           -ex - Setzt das Zeichen x (oder ein anderes angegebenes Zeichen.");
    puts("                 als Anfang einer Escape-Sequenz (normal \\).");
    puts("           -f<Dateiname> - Nimmt statt dem Filter FILTER.FLT, den Filter");
    puts("                           mit dem angegebenen Dateinamen als Filter.");
    puts("           -n - Unterdrckt die Generierung eines EOFs am Ende der Ausgabe.");
    return 1;
  }
  else
  {
    int firstread=0; /* Dateiname schon gelesen? */
    
    escape='\\';
    ignore=0;
    noeof=0;
    for(i=1;i<argc;i++)
    {
      if(*argv[i]=='-')
      {
        switch(argv[i][1])
        {
          case 'i':
            ignore=1;
            break;
          case 'e':
            escape=argv[i][2];
            break;
          case 'f':
            strcpy(filter,argv[i]+2);
            break;
          case 'n':
            noeof=1;
            break;
          default:
            puts("Ungltige Option");
        }
      }
      else
      {
        if(firstread)
          strcpy(outdev,argv[i]);
        else
        {
          firstread=1;
          strcpy(infname,argv[i]);
        }
      }
    }
    if(!*infname)
    {
      puts("Keine Datei angegeben");
      return 1;
    }
    else
    {
      if(!(in=fopen(infname,"rb")))
      {
        puts("Kann Datei nicht ffnen");
        return 1;
      }
      if(!(out=fopen(outdev,"wb")))
      {
        puts("Kann Ausgabe-(Datei) nicht ffnen");
        fclose(in);
        return 1;
      }
      if(!(flt=fopen(filter,"rb")))
      {
        puts("Kann Filterdatei nicht ffnen.");
        fclose(in);
        fclose(out);
        return 1;
      }
      filter_len=fread(buff,1,MAXFLTLEN,flt);
      fclose(flt);
      do_filter(in,out);
      fclose(out);
      fclose(in);
    }
  }
  return 0;
}
 